@@ -1,6 +1,7 @@
-from typing import Iterable, Iterator, Union
+from typing import List, Dict, Set, Iterable, Iterator, Union, Optional
 from pathlib import Path
 import numpy
+from numpy import ndarray
 import zlib
 import srsly
 from thinc.api import NumpyOps
@@ -8,9 +9,10 @@ from thinc.api import NumpyOps
 from .doc import Doc
 from ..vocab import Vocab
 from ..compat import copy_reg
-from ..attrs import SPACY, ORTH, intify_attr
+from ..attrs import SPACY, ORTH, intify_attr, IDS
 from ..errors import Errors
 from ..util import ensure_path, SimpleFrozenList
+from ._dict_proxies import SpanGroups
 
 # fmt: off
 ALL_ATTRS = ("ORTH", "NORM", "TAG", "HEAD", "DEP", "ENT_IOB", "ENT_TYPE", "ENT_KB_ID", "ENT_ID", "LEMMA", "MORPH", "POS", "SENT_START")
@@ -36,7 +38,7 @@ class DocBin:
         "spans": List[Dict[str, bytes]], # SpanGroups data for each doc
         "spaces": bytes, # Serialized numpy boolean array with spaces data
         "lengths": bytes, # Serialized numpy int32 array with the doc lengths
-        "strings": List[unicode] # List of unique strings in the token data
+        "strings": List[str] # List of unique strings in the token data
         "version": str, # DocBin version number
     }
 
@@ -64,17 +66,23 @@ class DocBin:
 
         DOCS: https://spacy.io/api/docbin#init
         """
-        attrs = sorted([intify_attr(attr) for attr in attrs])
+        int_attrs = [intify_attr(attr) for attr in attrs]
+        if None in int_attrs:
+            non_valid = [attr for attr in attrs if intify_attr(attr) is None]
+            raise KeyError(
+                Errors.E983.format(dict="attrs", key=non_valid, keys=IDS.keys())
+            ) from None
+        attrs = sorted(int_attrs)
         self.version = "0.1"
         self.attrs = [attr for attr in attrs if attr != ORTH and attr != SPACY]
         self.attrs.insert(0, ORTH)  # Ensure ORTH is always attrs[0]
-        self.tokens = []
-        self.spaces = []
-        self.cats = []
-        self.span_groups = []
-        self.user_data = []
-        self.flags = []
-        self.strings = set()
+        self.tokens: List[ndarray] = []
+        self.spaces: List[ndarray] = []
+        self.cats: List[Dict] = []
+        self.span_groups: List[bytes] = []
+        self.user_data: List[Optional[bytes]] = []
+        self.flags: List[Dict] = []
+        self.strings: Set[str] = set()
         self.store_user_data = store_user_data
         for doc in docs:
             self.add(doc)
@@ -103,12 +111,15 @@ class DocBin:
             self.strings.add(token.text)
             self.strings.add(token.tag_)
             self.strings.add(token.lemma_)
+            self.strings.add(token.norm_)
             self.strings.add(str(token.morph))
             self.strings.add(token.dep_)
             self.strings.add(token.ent_type_)
             self.strings.add(token.ent_kb_id_)
+            self.strings.add(token.ent_id_)
         self.cats.append(doc.cats)
-        self.user_data.append(srsly.msgpack_dumps(doc.user_data))
+        if self.store_user_data:
+            self.user_data.append(srsly.msgpack_dumps(doc.user_data))
         self.span_groups.append(doc.spans.to_bytes())
         for key, group in doc.spans.items():
             for span in group:
@@ -130,13 +141,14 @@ class DocBin:
         for i in range(len(self.tokens)):
             flags = self.flags[i]
             tokens = self.tokens[i]
-            spaces = self.spaces[i]
+            spaces: Optional[ndarray] = self.spaces[i]
             if flags.get("has_unknown_spaces"):
                 spaces = None
-            doc = Doc(vocab, words=tokens[:, orth_col], spaces=spaces)
-            doc = doc.from_array(self.attrs, tokens)
+            doc = Doc(vocab, words=tokens[:, orth_col], spaces=spaces)  # type: ignore
+            doc = doc.from_array(self.attrs, tokens)  # type: ignore
             doc.cats = self.cats[i]
-            if self.span_groups[i]:
+            # backwards-compatibility: may be b'' or serialized empty list
+            if self.span_groups[i] and self.span_groups[i] != SpanGroups._EMPTY_BYTES:
                 doc.spans.from_bytes(self.span_groups[i])
             else:
                 doc.spans.clear()
@@ -244,7 +256,10 @@ class DocBin:
         """
         path = ensure_path(path)
         with path.open("wb") as file_:
-            file_.write(self.to_bytes())
+            try:
+                file_.write(self.to_bytes())
+            except ValueError:
+                raise ValueError(Errors.E870)
 
     def from_disk(self, path: Union[str, Path]) -> "DocBin":
         """Load the DocBin from a file (typically called .spacy).
